@@ -25,6 +25,13 @@
 #include <config.h>
 #endif
 
+/**
+ * SECTION:glade-editor
+ * @Short_Description: A Widget to edit a #GladeWidget.
+ *
+ * This is the Glade Notebook containing all the controls needed to configure a #GladeWidget.
+ */
+
 #include <stdio.h>
 #include <string.h>
 #include <glib/gi18n-lib.h>
@@ -43,7 +50,6 @@
 #include "glade-project.h"
 #include "glade-utils.h"
 #include "glade-editor-property.h"
-#include "atk.xpm"
 
 enum
 {
@@ -185,7 +191,7 @@ glade_editor_notebook_page (GladeEditor *editor, const gchar *name)
 	GtkWidget     *sw;
 	GtkWidget     *label_widget;
 	GtkWidget     *image;
-	GdkPixbuf     *pixbuf;
+	static gchar  *path;
 	static gint page = 0;
 
 	/* alignment is needed to ensure property labels have some padding on the left */
@@ -195,19 +201,20 @@ glade_editor_notebook_page (GladeEditor *editor, const gchar *name)
 	/* construct tab label widget */
 	if (g_utf8_collate (name, _("Accessibility")) == 0)
 	{
-		pixbuf = gdk_pixbuf_new_from_xpm_data ((const char**) glade_atk_xpm);
+		path = g_build_filename (glade_app_get_pixmaps_dir (), "atk.png", NULL);
+		image = gtk_image_new_from_file (path);
 		label_widget = gtk_event_box_new ();
-		image = gtk_image_new_from_pixbuf (pixbuf);
 		gtk_container_add (GTK_CONTAINER (label_widget), image);
-		g_object_unref (G_OBJECT (pixbuf));
-		gtk_widget_show (image);
 		gtk_widget_show (label_widget);
+		gtk_widget_show (image);
 
 		glade_util_widget_set_tooltip (label_widget, name);
 	}
 	else
+	{
 		label_widget = gtk_label_new_with_mnemonic (name);
-
+	}
+	
 	/* configure page container */
 	if (g_utf8_collate (name, _("_Signals")) == 0)
 	{
@@ -243,13 +250,6 @@ glade_editor_on_reset_click (GtkButton *button,
 	glade_editor_reset_dialog (editor);
 }
 
-static void
-glade_editor_on_launch_click (GtkButton *button,
-			     GladeEditor *editor)
-{
-	glade_widget_launch_editor (editor->loaded_widget);
-}
-
 static void
 glade_editor_on_docs_click (GtkButton *button,
 			    GladeEditor *editor)
@@ -312,7 +312,6 @@ glade_editor_init (GladeEditor *editor)
 {
 	GtkSizeGroup *size_group;
 	GtkWidget *hbox;
-	GtkWidget *label;
 
 	editor->notebook     = gtk_notebook_new ();
 	editor->page_widget  = glade_editor_notebook_page (editor, _("_General"));
@@ -332,15 +331,6 @@ glade_editor_init (GladeEditor *editor)
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
 	gtk_box_pack_start (GTK_BOX (editor), hbox, FALSE, FALSE, 0);
 
-	/* Custom editor button */
-	editor->launch_button = gtk_button_new ();
-	label = gtk_label_new_with_mnemonic (_("_Edit..."));
-	gtk_misc_set_padding (GTK_MISC (label), 12, 0);
-	gtk_container_add (GTK_CONTAINER (editor->launch_button), label);
-	gtk_box_pack_start (GTK_BOX (hbox), editor->launch_button, FALSE, FALSE, 0);
-	g_signal_connect (G_OBJECT (editor->launch_button), "clicked",
-			  G_CALLBACK (glade_editor_on_launch_click), editor);
-
 	size_group = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
 
 	/* Reset button */
@@ -909,13 +899,6 @@ glade_editor_load_widget_real (GladeEditor *editor, GladeWidget *widget)
 	glade_editor_load_packing_page (editor, widget);
 	glade_signal_editor_load_widget (editor->signal_editor, widget);
 
-
-	if (widget && glade_widget_has_launcher (widget))
-		gtk_widget_show (editor->launch_button);
-	else
-		gtk_widget_hide (editor->launch_button);
-
-
 	/* we are just clearing, we are done */
 	if (widget == NULL)
 	{
@@ -980,6 +963,18 @@ glade_editor_refresh (GladeEditor *editor)
 	glade_editor_load_widget_real (editor, editor->loaded_widget);
 }
 
+static void
+query_dialog_style_set_cb (GtkWidget *dialog,
+			   GtkStyle  *previous_style,
+			   gpointer   user_data)
+{
+	gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 12);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 12);
+	gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->action_area), 0);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->action_area), 6);
+}
+
+
 gboolean
 glade_editor_query_dialog (GladeEditor *editor, GladeWidget *widget)
 {
@@ -1002,23 +997,30 @@ glade_editor_query_dialog (GladeEditor *editor, GladeWidget *widget)
 
 	g_free (title);
 
+	gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+						 GTK_RESPONSE_OK,
+						 GTK_RESPONSE_CANCEL,
+						 -1);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
 	table = glade_editor_get_table_from_class (editor,
 						   widget->adaptor,
 						   TABLE_TYPE_QUERY);
 
 
-	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
-
-	gtk_container_set_border_width (GTK_CONTAINER (table->table_widget), 6);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    table->table_widget,
-			    TRUE, TRUE, 4);
+			    FALSE, FALSE, 6);
 	for (list = table->properties; list; list = list->next)
 	{
 		property = list->data;
 		glade_editor_property_load_by_widget (property, widget);
 	}
 
+	g_signal_connect (dialog, "style-set", 
+			  G_CALLBACK (query_dialog_style_set_cb),
+			  NULL);
+
 	answer = gtk_dialog_run (GTK_DIALOG (dialog));
 
 	/*
